# Options:
#    install    - Installs the service.
#    start      - Starts the service.
#    stop       - Stops the service.
#    pause      - Pauses the service.
#    resume     - Resumes the service.
#    uninstall  - Uninstalls the service.
#    delete     - Same as uninstall.
require 'win32/service'
require 'rbconfig'



class WindowsServiceController
  include RbConfig
  include Win32

  SERVICE_NAME = 'cantemo_portal_watch_folder_agent'
  SERVICE_DISPLAYNAME = 'Cantemo Portal Watch Folder Agent'

  class << self


    def run(command = nil, args = { })
      # You must provide at least one argument.
      raise ArgumentError, 'No argument provided.' unless command
      unless Service.exists?(SERVICE_NAME) || command == 'install'
        puts 'Service is not installed.'
        return
      end

      case command
      when 'start'; self.start
      when 'stop'; self.stop
      when 'pause'; self.pause
      when 'resume'; self.resume
      when 'status'; self.status
      when 'install'; self.install(args)
      when 'uninstall', 'delete'; self.uninstall
      else
        raise ArgumentError, 'unknown option: ' + command
      end
    end

    def install(args = {})
      # Quote the full path to deal with possible spaces in the path name.
      ruby = File.join(CONFIG['bindir'], CONFIG['ruby_install_name']).tr('/', '\\')
      path = __FILE__.tr('/', '\\')
      cmd = %Q("#{ruby}" "#{path}")

      install_args = {
          :service_name     => SERVICE_NAME,
          :display_name     => SERVICE_DISPLAYNAME,
          :binary_path_name => cmd
      }.merge!(args)

      Service.new(install_args)
      puts "Service `#{SERVICE_NAME}` installed"
    end

    def uninstall
      if Service.status(SERVICE_NAME).current_state != 'stopped'
        Service.stop(SERVICE_NAME)
      end
      while Service.status(SERVICE_NAME).current_state != 'stopped'
        puts "One moment... #{Service.status(SERVICE_NAME).current_state}"
        sleep 1
      end
      Service.delete(SERVICE_NAME)
      puts "Service #{SERVICE_NAME} deleted"
    end

    def start
      if Service.status(SERVICE_NAME).current_state != 'running'
        # Service.start(SERVICE_NAME, nil, args)
        Service.start(SERVICE_NAME)
        while Service.status(SERVICE_NAME).current_state != 'running'
          puts "One moment... #{Service.status(SERVICE_NAME).current_state}"
          sleep 1
        end
        puts "Service `#{SERVICE_NAME}` started"
      else
        puts 'Already running'
      end
    end

    def status
      puts "Service `#{SERVICE_NAME}` #{Service.status(SERVICE_NAME).current_state}"
    end

    def stop
      if Service.status(SERVICE_NAME).current_state != 'stopped'
        Service.stop(SERVICE_NAME)
        while Service.status(SERVICE_NAME).current_state != 'stopped'
          puts "One moment... #{Service.status(SERVICE_NAME).current_state}"
          sleep 1
        end
        puts "Service `#{SERVICE_NAME}` stopped"
      else
        puts 'Already stopped'
      end
    end

    def pause
      if Service.status(SERVICE_NAME).current_state != 'paused'
        Service.pause(SERVICE_NAME)
        while Service.status(SERVICE_NAME).current_state != 'paused'
          puts "One moment... #{Service.status(SERVICE_NAME).current_state}"
          sleep 1
        end
        puts "Service `#{SERVICE_NAME}` paused"
      else
        puts 'Already paused'
      end
    end

    def resume
      if Service.status(SERVICE_NAME).current_state != 'running'
        Service.resume(SERVICE_NAME)
        while Service.status(SERVICE_NAME).current_state != 'running'
          puts "One moment... #{Service.status(SERVICE_NAME).current_state}"
          sleep 1
        end
        puts "Service `#{SERVICE_NAME}` resumed"
      else
        puts 'Already running'
      end
    end

  end

end


