module Envoi
  module Mam
    class Agent
      class WatchFolderUtility
        class Foreman

          AWF = Envoi::Aspera::WatchService::WatchFolder
          LWF = Envoi::WatchFolderUtility::WatchFolder::Listen

          attr_accessor :logger, :agent, :config, :watch_folders

          def initialize(args = { })
            initialize_logger(args)

            @agent = Envoi::Mam::Cantemo::Agent.load_from_config_file(args)
            @config = agent.config
            cantemo_config = config[:cantemo] || config['cantemo']
            watch_folder_defs = cantemo_config[:watch_folders] || cantemo_config['watch_folders']

            @ignored_file_paths_by_watch_folder = Hash.new { |h, k| h[k] = [] }



            @watch_folders = AWF.process_watch_folder_defs(watch_folder_defs)
          end

          def self.initialize_watch_folder(watch_folder_def)
            case  (watch_folder_def['handler'] || '').downcase.gsub(/[-_ ]/, '')
              when 'asperawatch'
                wf_class = Envoi::Aspera::WatchService::WatchFolder
            end
            wf_class.new(watch_folder_def)
          end

          def self.process_watch_folder_def(watch_folder_def)
            initialize_watch_folder(watch_folder_def)
          end

          def self.process_watch_folder_defs(watch_folder_defs)
            if watch_folder_defs.is_a?(Array)
              watch_folder_defs.map { |watch_folder_def| process_watch_folder_def(watch_folder_def) }
            elsif watch_folder_defs.is_a?(Hash)
              watch_folder_defs.map do |name, watch_folder_def|
                watch_folder_def['path'] ||= name; process_watch_folder_def(watch_folder_def)
              end
            else
              raise "Unhandled format: #{watch_folder_defs.class.name}"
            end
          end

          def self.poll_watch_folder(watch_folder, &block)
            watch_folder.poll(&block)
          end

          def self.poll_watch_folders(watch_folders, &block)
            watch_folders.each { |watch_folder| poll_watch_folder(watch_folder, &block) }
          end

          def self.run_once(watch_folders, &block)
            poll_watch_folders(watch_folders, &block)
          end

          # Foreman
        end

        # WatchFolderUtility
      end

      # Agent
    end

    # Mam
  end

  # Envoi
end